#include <sys/types.h>

#include <tcl.h>
#include <tk.h>
#include <X11/Xutil.h>          /* for XVisualInfo */

#include <sys/ipc.h>            /* for share memory */
#include <sys/shm.h>
#include <X11/extensions/XShm.h>

#include "message.h"
#include "control.h"

#ifndef FLY_H
#define FLY_H

#define TRUE                1
#define FALSE               0

#define SET_none            0
#define SET_ExposedViewer   1
#define SET_Timer           2


struct ParentWindow {
   /* Global data */
   int             synchronize;
   char           *name;
   char           *displayname;
   char           *geometry;

   Tcl_Interp     *interp;

   Tk_Window       tkwin;
   Display        *display;
   Visual         *visual;      /* X visual for the display window */
   XVisualInfo     visual_info; /* information about visual */

   int             width;       /* actual width of window area */
   int             height;      /* actual height of window area */

   int             depth;       /* bits/pixel for the window */

   int             ncolor;
   Colormap        colormap;    /* current colormap for this window */
   XColor         *colors;
   unsigned long  *pixels;
#if 0
   char           *ownCmap;     /* string saying whether to use private cmap */
   Tk_Uid          palette;     /* specifies color allocations */
   ColorTable     *colors;      /* points to info about mapping colours */
   Colormap        cmap;        /* current colormap for this window */
   Colormap        defaultCmap; /* default colormap in this window */
#endif
};

typedef struct ParentWindow ParentWindow;


struct Viewer {
   /* Global rootwindow info */
   ParentWindow    *parentwindow;

   Tcl_Interp     *interp;

   Tk_Window       tkwin;
   Window          root;
   Display        *display;
   Visual         *visual;      /* X visual for the display window */
   XVisualInfo     visual_info; /* information about visual */

   Tk_Uid          palette;     /* specifies color allocations */

   int             ncolor;
   Colormap        colormap;    /* current colormap for this window */
   XColor         *colors;
   unsigned long  *pixels;

   GC              blankGC;     /* GC for filling in blankColor */

   Region          region;      /* bits we need to redraw */

   Tk_TimerToken   timer;

   XShmSegmentInfo imageshminfo;/* Segment info.  */
   XImage         *image;       /* X11 image */

   XShmSegmentInfo pixmapshminfo;/* Segment info.  */
   Pixmap          pixelmap;      /* backing pixmap */

   int             depth;       /* bits/pixel for the image */
   int             width;       /* actual width of image display area */
   int             height;      /* actual height of image display area */
   Region          imageregion; /* bits we need to redraw */
   int             imageSize;   /* # bytes available at image->data */
   XColor         *white;
   XColor         *black;

   char           *datafilename;
   FILE           *datafile;

   Comm            comm;

   pid_t        ppid;  /* Parent pid */
   pid_t        pid;  /* Child pid */

   int status;

   /* view matrix data */
   double   View_matrix[4][4];
   double   Perspective_matrix[4][4];
   double   Transformation_matrix[4][4];
   double   Viewport_matrix[4][4];

   /* Image data */
   XShmSegmentInfo lineshminfo;/* Segment info.  */
   struct LineData *linedata;

   /* only exists in the parent */
   Control  *control;

#if 0
   /* only exists in the child */
   Object  *object;
#endif
};

typedef struct Viewer Viewer;

struct LineData {
   int nsegments;
   XSegment segments[1000];
};

typedef struct LineData LineData;

#endif

int viewercmd(ClientData clientData, Tcl_Interp * interp, int argc,
              char *argv[]);
void child_main(Viewer *viewer);
void UpdateViewer(ClientData clientdata);
void compute_view_info(ClientData clientdata);

